﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using Microsoft.SharePoint;

namespace Erucy.Utilities
{
    public class SPStorageMetricItem
    {
        public enum MetricType
        {
            Invalid = -1,
            File = 0,
            Folder = 1,
            Web = 2
        }

        public string DisplayName { get; private set; }
        public Guid Id { get; private set; }
        public DateTime LastModified { get; private set; }
        public long? TotalSize { get; private set; }
        public MetricType Type { get; private set; }
        public string Url { get; private set; }
        public Guid WebId { get; private set; }

        public SPStorageMetricItem(object metricObj)
        {
            this.DisplayName = (string)GetPropValue(metricObj, "DisplayName");
            this.Id = (Guid)GetPropValue(metricObj, "Id");
            this.LastModified = (DateTime)GetPropValue(metricObj, "LastModified");
            this.TotalSize = (long?)GetPropValue(metricObj, "TotalSize");
            this.Url = (string)GetPropValue(metricObj, "Url");
            this.WebId = (Guid)GetPropValue(metricObj, "WebId");

            string itemType = Convert.ToString(GetPropValue(metricObj, "Type"));
            switch (itemType)
            {
                case "File":
                    this.Type = MetricType.File;
                    break;
                case "Folder":
                    this.Type = MetricType.Folder;
                    break;
                case "Web":
                    this.Type = MetricType.Web;
                    break;
                default:
                    this.Type = MetricType.Invalid;
                    break;
            }
        }

        private object GetPropValue(object obj, string propertyName)
        {
            PropertyInfo propInfo = obj.GetType().GetProperty(propertyName,
                BindingFlags.GetProperty | BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.ExactBinding);
            return propInfo.GetValue(obj, null);
        }
    }

    public static class StorageMetricExtension
    {
        public static List<SPStorageMetricItem> GetStorageMetrics(this SPSite site, string root, int orderBy, bool isAscending)
        {
            MethodInfo getMetricMethod = typeof(SPSite).GetMethod("GetStorageMetrics", 
                BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.ExactBinding);
            object metricResultObj = getMetricMethod.Invoke(site, new object[] { root, orderBy, isAscending });

            PropertyInfo childrenProperty = metricResultObj.GetType().GetProperty("Children", 
                BindingFlags.GetProperty | BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.ExactBinding);
            IList rawMetricItems = childrenProperty.GetValue(metricResultObj, null) as IList;

            List<SPStorageMetricItem> result = new List<SPStorageMetricItem>(rawMetricItems.Count);
            foreach (object metricObj in rawMetricItems)
            {
                result.Add(new SPStorageMetricItem(metricObj));
            }
            return result;
        }
    }
}
